@@ -1,4 +1,5 @@
-# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
+# -*- coding: utf-8 -*-
+# © Copyright EnterpriseDB UK Limited 2013-2021
 #
 # This file is part of Barman.
 #
@@ -17,6 +18,7 @@
 
 import ast
 import collections
+import inspect
 import logging
 import os
 
@@ -24,11 +26,15 @@ import dateutil.parser
 import dateutil.tz
 
 from barman import xlog
-from barman.compression import identify_compression
+from barman.exceptions import BackupInfoBadInitialisation
+from barman.utils import fsync_dir
 
-# create a namedtuple object called Tablespace with 'name' 'oid' and 'location'
+# Named tuple representing a Tablespace with 'name' 'oid' and 'location'
 # as property.
-Tablespace = collections.namedtuple('Tablespace', 'name oid location')
+Tablespace = collections.namedtuple("Tablespace", "name oid location")
+
+# Named tuple representing a file 'path' with an associated 'file_type'
+TypedFile = collections.namedtuple("ConfFile", "file_type path")
 
 _logger = logging.getLogger(__name__)
 
@@ -92,7 +98,6 @@ def load_datetime_tz(time_str):
 
 
 class Field(object):
-
     def __init__(self, name, dump=None, load=None, default=None, doc=None):
         """
         Field descriptor to be used with a FieldListFile subclass.
@@ -125,12 +130,12 @@ class Field(object):
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
-        if not hasattr(obj, '_fields'):
+        if not hasattr(obj, "_fields"):
             obj._fields = {}
         return obj._fields.setdefault(self.name, self.default)
 
     def __set__(self, obj, value):
-        if not hasattr(obj, '_fields'):
+        if not hasattr(obj, "_fields"):
             obj._fields = {}
         obj._fields[self.name] = value
 
@@ -146,7 +151,7 @@ class Field(object):
 
 class FieldListFile(object):
 
-    __slots__ = ('_fields', 'filename')
+    __slots__ = ("_fields", "filename")
 
     def __init__(self, **kwargs):
         """
@@ -171,7 +176,7 @@ class FieldListFile(object):
             if isinstance(field, Field):
                 setattr(self, name, kwargs[name])
             else:
-                raise AttributeError('unknown attribute %s' % name)
+                raise AttributeError("unknown attribute %s" % name)
 
     @classmethod
     def from_meta_file(cls, filename):
@@ -205,22 +210,29 @@ class FieldListFile(object):
         else:
             filename = filename or self.filename
             if filename:
-                info = open(filename, 'w')
+                info = open(filename + ".tmp", "wb")
             else:
                 info = None
 
         if not info:
             raise ValueError(
-                'either a valid filename or a file_object must be specified')
+                "either a valid filename or a file_object must be specified"
+            )
 
-        with info:
-            for name in sorted(vars(type(self))):
-                field = getattr(type(self), name)
+        try:
+            for name, field in sorted(inspect.getmembers(type(self))):
                 value = getattr(self, name, None)
                 if isinstance(field, Field):
                     if callable(field.to_str):
                         value = field.to_str(value)
-                    info.write("%s=%s\n" % (name, value))
+                    info.write(("%s=%s\n" % (name, value)).encode("UTF-8"))
+        finally:
+            if not file_object:
+                info.close()
+
+        if not file_object:
+            os.rename(filename + ".tmp", filename)
+            fsync_dir(os.path.normpath(os.path.dirname(filename)))
 
     def load(self, filename=None, file_object=None):
         """
@@ -241,14 +253,13 @@ class FieldListFile(object):
         if file_object:
             info = file_object
         elif filename:
-            info = open(filename, 'r')
+            info = open(filename, "rb")
         else:
-            raise ValueError(
-                'either filename or file_object must be specified')
+            raise ValueError("either filename or file_object must be specified")
 
         # detect the filename if a file_object is passed
         if not filename and file_object:
-            if hasattr(file_object, 'name'):
+            if hasattr(file_object, "name"):
                 filename = file_object.name
 
         # canonicalize filename
@@ -256,27 +267,29 @@ class FieldListFile(object):
             self.filename = os.path.abspath(filename)
         else:
             self.filename = None
-            filename = '<UNKNOWN>'  # This is only for error reporting
+            filename = "<UNKNOWN>"  # This is only for error reporting
 
         with info:
             for line in info:
+                line = line.decode("UTF-8")
                 # skip spaces and comments
-                if line.isspace() or line.rstrip().startswith('#'):
+                if line.isspace() or line.rstrip().startswith("#"):
                     continue
 
                 # parse the line of form "key = value"
                 try:
-                    name, value = [x.strip() for x in line.split('=', 1)]
+                    name, value = [x.strip() for x in line.split("=", 1)]
                 except ValueError:
-                    raise ValueError('invalid line %s in file %s' % (
-                        line.strip(), filename))
+                    raise ValueError(
+                        "invalid line %s in file %s" % (line.strip(), filename)
+                    )
 
                 # use the from_str function to parse the value
                 field = getattr(type(self), name, None)
-                if value == 'None':
+                if value == "None":
                     value = None
                 elif isinstance(field, Field) and callable(field.from_str):
-                        value = field.from_str(value)
+                    value = field.from_str(value)
                 setattr(self, name, value)
 
     def items(self):
@@ -285,8 +298,7 @@ class FieldListFile(object):
 
         If a filed has a dump function defined, it will be used.
         """
-        for name in sorted(vars(type(self))):
-            field = getattr(type(self), name)
+        for name, field in sorted(inspect.getmembers(type(self))):
             value = getattr(self, name, None)
             if isinstance(field, Field):
                 if callable(field.to_str):
@@ -296,7 +308,8 @@ class FieldListFile(object):
     def __repr__(self):
         return "%s(%s)" % (
             self.__class__.__name__,
-            ', '.join(['%s=%r' % x for x in self.items()]))
+            ", ".join(["%s=%r" % x for x in self.items()]),
+        )
 
 
 class WalFileInfo(FieldListFile):
@@ -304,16 +317,19 @@ class WalFileInfo(FieldListFile):
     Metadata of a WAL file.
     """
 
-    __slots__ = ('orig_filename',)
+    __slots__ = ("orig_filename",)
 
-    name = Field('name', doc='base name of WAL file')
-    size = Field('size', load=int, doc='WAL file size after compression')
-    time = Field('time', load=float, doc='WAL file modification time '
-                                         '(seconds since epoch)')
-    compression = Field('compression', doc='compression type')
+    name = Field("name", doc="base name of WAL file")
+    size = Field("size", load=int, doc="WAL file size after compression")
+    time = Field(
+        "time", load=float, doc="WAL file modification time (seconds since epoch)"
+    )
+    compression = Field("compression", doc="compression type")
 
     @classmethod
-    def from_file(cls, filename, default_compression=None, **kwargs):
+    def from_file(
+        cls, filename, compression_manager=None, unidentified_compression=None, **kwargs
+    ):
         """
         Factory method to generate a WalFileInfo from a WAL file.
 
@@ -322,16 +338,20 @@ class WalFileInfo(FieldListFile):
         an AttributeError exception is raised.
 
         :param str filename: the file to inspect
-        :param str default_compression: the compression to set if
-            the current schema is not identifiable.
+        :param Compressionmanager compression_manager: a compression manager
+            which will be used to identify the compression
+        :param str unidentified_compression: the compression to set if
+            the current schema is not identifiable
         """
         stat = os.stat(filename)
-        kwargs.setdefault('name', os.path.basename(filename))
-        kwargs.setdefault('size', stat.st_size)
-        kwargs.setdefault('time', stat.st_mtime)
-        if 'compression' not in kwargs:
-            kwargs['compression'] = identify_compression(filename) \
-                or default_compression
+        kwargs.setdefault("name", os.path.basename(filename))
+        kwargs.setdefault("size", stat.st_size)
+        kwargs.setdefault("time", stat.st_mtime)
+        if "compression" not in kwargs:
+            kwargs["compression"] = (
+                compression_manager.identify_compression(filename)
+                or unidentified_compression
+            )
         obj = cls(**kwargs)
         obj.filename = "%s.meta" % filename
         obj.orig_filename = filename
@@ -341,11 +361,7 @@ class WalFileInfo(FieldListFile):
         """
         Format the content of this object as a xlogdb line.
         """
-        return "%s\t%s\t%s\t%s\n" % (
-            self.name,
-            self.size,
-            self.time,
-            self.compression)
+        return "%s\t%s\t%s\t%s\n" % (self.name, self.size, self.time, self.compression)
 
     @classmethod
     def from_xlogdb_line(cls, line):
@@ -365,12 +381,11 @@ class WalFileInfo(FieldListFile):
             except ValueError:
                 raise ValueError("cannot parse line: %r" % (line,))
         # The to_xlogdb_line method writes None values as literal 'None'
-        if compression == 'None':
+        if compression == "None":
             compression = None
         size = int(size)
         time = float(time)
-        return cls(name=name, size=size, time=time,
-                   compression=compression)
+        return cls(name=name, size=size, time=time, compression=compression)
 
     def to_json(self):
         """
@@ -393,67 +408,204 @@ class WalFileInfo(FieldListFile):
         return os.path.join(server.config.wals_directory, self.relpath())
 
 
-class UnknownBackupIdException(Exception):
-    """
-    The searched backup_id doesn't exists
-    """
-
-
-class BackupInfoBadInitialisation(Exception):
-    """
-    Exception for a bad initialization error
-    """
-
-
 class BackupInfo(FieldListFile):
 
     #: Conversion to string
-    EMPTY = 'EMPTY'
-    STARTED = 'STARTED'
-    FAILED = 'FAILED'
-    DONE = 'DONE'
-    STATUS_ALL = (EMPTY, STARTED, DONE, FAILED)
-    STATUS_NOT_EMPTY = (STARTED, DONE, FAILED)
-    STATUS_ARCHIVING = (STARTED, DONE)
+    EMPTY = "EMPTY"
+    STARTED = "STARTED"
+    FAILED = "FAILED"
+    WAITING_FOR_WALS = "WAITING_FOR_WALS"
+    DONE = "DONE"
+    SYNCING = "SYNCING"
+    STATUS_COPY_DONE = (WAITING_FOR_WALS, DONE)
+    STATUS_ALL = (EMPTY, STARTED, WAITING_FOR_WALS, DONE, SYNCING, FAILED)
+    STATUS_NOT_EMPTY = (STARTED, WAITING_FOR_WALS, DONE, SYNCING, FAILED)
+    STATUS_ARCHIVING = (STARTED, WAITING_FOR_WALS, DONE, SYNCING)
 
     #: Status according to retention policies
-    OBSOLETE = 'OBSOLETE'
-    VALID = 'VALID'
-    POTENTIALLY_OBSOLETE = 'OBSOLETE*'
-    NONE = '-'
-    RETENTION_STATUS = (OBSOLETE, VALID, POTENTIALLY_OBSOLETE, NONE)
+    OBSOLETE = "OBSOLETE"
+    VALID = "VALID"
+    POTENTIALLY_OBSOLETE = "OBSOLETE*"
+    NONE = "-"
+    KEEP_FULL = "KEEP:FULL"
+    KEEP_STANDALONE = "KEEP:STANDALONE"
+    RETENTION_STATUS = (
+        OBSOLETE,
+        VALID,
+        POTENTIALLY_OBSOLETE,
+        KEEP_FULL,
+        KEEP_STANDALONE,
+        NONE,
+    )
 
-    version = Field('version', load=int)
-    pgdata = Field('pgdata')
+    version = Field("version", load=int)
+    pgdata = Field("pgdata")
     # Parse the tablespaces as a literal Python list of namedtuple
     # Output the tablespaces as a literal Python list of tuple
-    tablespaces = Field('tablespaces', load=load_tablespace_list,
-                        dump=output_tablespace_list)
+    tablespaces = Field(
+        "tablespaces", load=load_tablespace_list, dump=output_tablespace_list
+    )
     # Timeline is an integer
-    timeline = Field('timeline', load=int)
-    begin_time = Field('begin_time', load=load_datetime_tz)
-    begin_xlog = Field('begin_xlog')
-    begin_wal = Field('begin_wal')
-    begin_offset = Field('begin_offset', load=int)
-    size = Field('size', load=int)
-    deduplicated_size = Field('deduplicated_size', load=int)
-    end_time = Field('end_time', load=load_datetime_tz)
-    end_xlog = Field('end_xlog')
-    end_wal = Field('end_wal')
-    end_offset = Field('end_offset', load=int)
-    status = Field('status', default=EMPTY)
-    server_name = Field('server_name')
-    error = Field('error')
-    mode = Field('mode')
-    config_file = Field('config_file')
-    hba_file = Field('hba_file')
-    ident_file = Field('ident_file')
-    included_files = Field('included_files',
-                           load=ast.literal_eval, dump=null_repr)
-    backup_label = Field('backup_label', load=ast.literal_eval, dump=null_repr)
+    timeline = Field("timeline", load=int)
+    begin_time = Field("begin_time", load=load_datetime_tz)
+    begin_xlog = Field("begin_xlog")
+    begin_wal = Field("begin_wal")
+    begin_offset = Field("begin_offset", load=int)
+    size = Field("size", load=int)
+    deduplicated_size = Field("deduplicated_size", load=int)
+    end_time = Field("end_time", load=load_datetime_tz)
+    end_xlog = Field("end_xlog")
+    end_wal = Field("end_wal")
+    end_offset = Field("end_offset", load=int)
+    status = Field("status", default=EMPTY)
+    server_name = Field("server_name")
+    error = Field("error")
+    mode = Field("mode")
+    config_file = Field("config_file")
+    hba_file = Field("hba_file")
+    ident_file = Field("ident_file")
+    included_files = Field("included_files", load=ast.literal_eval, dump=null_repr)
+    backup_label = Field("backup_label", load=ast.literal_eval, dump=null_repr)
+    copy_stats = Field("copy_stats", load=ast.literal_eval, dump=null_repr)
+    xlog_segment_size = Field(
+        "xlog_segment_size", load=int, default=xlog.DEFAULT_XLOG_SEG_SIZE
+    )
+    systemid = Field("systemid")
 
-    __slots__ = ('server', 'config', 'backup_manager',
-                 'backup_id', 'backup_version')
+    __slots__ = "backup_id", "backup_version"
+
+    def __init__(self, backup_id, **kwargs):
+        """
+        Stores meta information about a single backup
+
+        :param str,None backup_id:
+        """
+        self.backup_version = 2
+        self.backup_id = backup_id
+        super(BackupInfo, self).__init__(**kwargs)
+
+    def get_required_wal_segments(self):
+        """
+        Get the list of required WAL segments for the current backup
+        """
+        return xlog.generate_segment_names(
+            self.begin_wal, self.end_wal, self.version, self.xlog_segment_size
+        )
+
+    def get_external_config_files(self):
+        """
+        Identify all the configuration files that reside outside the PGDATA.
+
+        Returns a list of TypedFile objects.
+
+        :rtype: list[TypedFile]
+        """
+
+        config_files = []
+        for file_type in ("config_file", "hba_file", "ident_file"):
+            config_file = getattr(self, file_type, None)
+            if config_file:
+                # Consider only those that reside outside of the original
+                # PGDATA directory
+                if config_file.startswith(self.pgdata):
+                    _logger.debug(
+                        "Config file '%s' already in PGDATA",
+                        config_file[len(self.pgdata) + 1 :],
+                    )
+                    continue
+                config_files.append(TypedFile(file_type, config_file))
+        # Check for any include directives in PostgreSQL configuration
+        # Currently, include directives are not supported for files that
+        # reside outside PGDATA. These files must be manually backed up.
+        # Barman will emit a warning and list those files
+        if self.included_files:
+            for included_file in self.included_files:
+                if not included_file.startswith(self.pgdata):
+                    config_files.append(TypedFile("include", included_file))
+        return config_files
+
+    def set_attribute(self, key, value):
+        """
+        Set a value for a given key
+        """
+        setattr(self, key, value)
+
+    def to_dict(self):
+        """
+        Return the backup_info content as a simple dictionary
+
+        :return dict:
+        """
+        result = dict(self.items())
+        result.update(
+            backup_id=self.backup_id,
+            server_name=self.server_name,
+            mode=self.mode,
+            tablespaces=self.tablespaces,
+            included_files=self.included_files,
+            copy_stats=self.copy_stats,
+        )
+        return result
+
+    def to_json(self):
+        """
+        Return an equivalent dictionary that uses only json-supported types
+        """
+        data = self.to_dict()
+        # Convert fields which need special types not supported by json
+        if data.get("tablespaces") is not None:
+            data["tablespaces"] = [list(item) for item in data["tablespaces"]]
+        if data.get("begin_time") is not None:
+            data["begin_time"] = data["begin_time"].ctime()
+        if data.get("end_time") is not None:
+            data["end_time"] = data["end_time"].ctime()
+        return data
+
+    @classmethod
+    def from_json(cls, server, json_backup_info):
+        """
+        Factory method that builds a BackupInfo object
+        from a json dictionary
+
+        :param barman.Server server: the server related to the Backup
+        :param dict json_backup_info: the data set containing values from json
+        """
+        data = dict(json_backup_info)
+        # Convert fields which need special types not supported by json
+        if data.get("tablespaces") is not None:
+            data["tablespaces"] = [
+                Tablespace._make(item) for item in data["tablespaces"]
+            ]
+        if data.get("begin_time") is not None:
+            data["begin_time"] = load_datetime_tz(data["begin_time"])
+        if data.get("end_time") is not None:
+            data["end_time"] = load_datetime_tz(data["end_time"])
+        # Instantiate a BackupInfo object using the converted fields
+        return cls(server, **data)
+
+    def pg_major_version(self):
+        """
+        Returns the major version of the PostgreSQL instance from which the
+        backup was made taking into account the change in versioning scheme
+        between PostgreSQL < 10.0 and PostgreSQL >= 10.0.
+        """
+        major = int(self.version / 10000)
+        if major < 10:
+            minor = int(self.version / 100 % 100)
+            return "%d.%d" % (major, minor)
+        else:
+            return str(major)
+
+    def wal_directory(self):
+        """
+        Returns "pg_wal" (v10 and above) or "pg_xlog" (v9.6 and below) based on
+        the Postgres version represented by this backup
+        """
+        return "pg_wal" if self.version >= 100000 else "pg_xlog"
+
+
+class LocalBackupInfo(BackupInfo):
+    __slots__ = "server", "config", "backup_manager"
 
     def __init__(self, server, info_file=None, backup_id=None, **kwargs):
         """
@@ -467,18 +619,19 @@ class BackupInfo(FieldListFile):
         """
         # Initialises the attributes for the object
         # based on the predefined keys
-        super(BackupInfo, self).__init__(**kwargs)
+        super(LocalBackupInfo, self).__init__(backup_id=backup_id, **kwargs)
 
         self.server = server
         self.config = server.config
         self.backup_manager = self.server.backup_manager
         self.server_name = self.config.name
-        self.mode = self.backup_manager.name
+        self.mode = self.backup_manager.mode
         if backup_id:
             # Cannot pass both info_file and backup_id
             if info_file:
                 raise BackupInfoBadInitialisation(
-                    'both info_file and backup_id parameters are set')
+                    "both info_file and backup_id parameters are set"
+                )
             self.backup_id = backup_id
             self.filename = self.get_filename()
             # Check if a backup info file for a given server and a given ID
@@ -486,7 +639,7 @@ class BackupInfo(FieldListFile):
             if os.path.exists(self.filename):
                 self.load(filename=self.filename)
         elif info_file:
-            if hasattr(info_file, 'read'):
+            if hasattr(info_file, "read"):
                 # We have been given a file-like object
                 self.load(file_object=info_file)
             else:
@@ -495,42 +648,38 @@ class BackupInfo(FieldListFile):
             self.backup_id = self.detect_backup_id()
         elif not info_file:
             raise BackupInfoBadInitialisation(
-                'backup_id and info_file parameters are both unset')
+                "backup_id and info_file parameters are both unset"
+            )
         # Manage backup version for new backup structure
-        self.backup_version = 2
         try:
             # the presence of pgdata directory is the marker of version 1
             if self.backup_id is not None and os.path.exists(
-                    os.path.join(self.get_basebackup_directory(), 'pgdata')):
+                os.path.join(self.get_basebackup_directory(), "pgdata")
+            ):
                 self.backup_version = 1
         except Exception as e:
-            _logger.warning("Error detecting backup_version, "
-                            "use default: 2. Failure reason: %s", e)
-
-    def get_required_wal_segments(self):
-        """
-        Get the list of required WAL segments for the current backup
-        """
-        return xlog.enumerate_segments(self.begin_wal, self.end_wal,
-                                       self.version)
+            _logger.warning(
+                "Error detecting backup_version, use default: 2. Failure reason: %s",
+                e,
+            )
 
     def get_list_of_files(self, target):
         """
         Get the list of files for the current backup
         """
         # Walk down the base backup directory
-        if target in ('data', 'standalone', 'full'):
+        if target in ("data", "standalone", "full"):
             for root, _, files in os.walk(self.get_basebackup_directory()):
                 for f in files:
                     yield os.path.join(root, f)
-        if target in 'standalone':
+        if target in "standalone":
             # List all the WAL files for this backup
             for x in self.get_required_wal_segments():
                 yield self.server.get_wal_full_path(x)
-        if target in ('wal', 'full'):
+        if target in ("wal", "full"):
             for wal_info in self.server.get_wal_until_next_backup(
-                    self,
-                    include_history=True):
+                self, include_history=True
+            ):
                 yield wal_info.fullpath(self.server)
 
     def detect_backup_id(self):
@@ -547,8 +696,7 @@ class BackupInfo(FieldListFile):
         Get the default filename for the backup.info file based on
         backup ID and server directory for base backups
         """
-        return os.path.join(self.config.basebackups_directory,
-                            self.backup_id)
+        return os.path.join(self.config.basebackups_directory, self.backup_id)
 
     def get_data_directory(self, tablespace_oid=None):
         """
@@ -557,17 +705,23 @@ class BackupInfo(FieldListFile):
         If tablespace_oid is passed, build the path to the tablespace
         base directory, according with the backup version
 
-        :param str tablespace_oid: the oid of a valid tablespace
+        :param int tablespace_oid: the oid of a valid tablespace
         """
         # Check if a tablespace oid is passed and if is a valid oid
-        if tablespace_oid is not None and (
-                self.tablespaces is None or
-                all(str(tablespace_oid) != str(tablespace.oid)
-                    for tablespace in self.tablespaces)):
-            raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+        if tablespace_oid is not None:
+            if self.tablespaces is None:
+                raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+
+            invalid_oid = all(
+                str(tablespace_oid) != str(tablespace.oid)
+                for tablespace in self.tablespaces
+            )
+            if invalid_oid:
+                raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+
         # Build the requested path according to backup_version value
         path = [self.get_basebackup_directory()]
-        # Check te version of the backup
+        # Check the version of the backup
         if self.backup_version == 2:
             # If an oid has been provided, we are looking for a tablespace
             if tablespace_oid is not None:
@@ -575,14 +729,14 @@ class BackupInfo(FieldListFile):
                 path.append(str(tablespace_oid))
             else:
                 # Looking for the data dir
-                path.append('data')
+                path.append("data")
         else:
             # Backup v1, use pgdata as base
-            path.append('pgdata')
+            path.append("pgdata")
             # If a oid has been provided, we are looking for a tablespace.
             if tablespace_oid is not None:
                 # Append the path to pg_tblspc/oid folder inside pgdata
-                path.extend(('pg_tblspc', str(tablespace_oid)))
+                path.extend(("pg_tblspc", str(tablespace_oid)))
         # Return the built path
         return os.path.join(*path)
 
@@ -591,13 +745,7 @@ class BackupInfo(FieldListFile):
         Get the default filename for the backup.info file based on
         backup ID and server directory for base backups
         """
-        return os.path.join(self.get_basebackup_directory(), 'backup.info')
-
-    def set_attribute(self, key, value):
-        """
-        Set a value for a given key
-        """
-        setattr(self, key, value)
+        return os.path.join(self.get_basebackup_directory(), "backup.info")
 
     def save(self, filename=None, file_object=None):
         if not file_object:
@@ -606,52 +754,4 @@ class BackupInfo(FieldListFile):
             dir_name = os.path.dirname(filename)
             if not os.path.exists(dir_name):
                 os.makedirs(dir_name)
-        super(BackupInfo, self).save(filename=filename,
-                                     file_object=file_object)
-
-    def to_dict(self):
-        """
-        Return the backup_info content as a simple dictionary
-
-        :return dict:
-        """
-        result = dict(self.items())
-        result.update(backup_id=self.backup_id, server_name=self.server_name,
-                      mode=self.mode, tablespaces=self.tablespaces)
-        return result
-
-    def to_json(self):
-        """
-        Return an equivalent dictionary that uses only json-supported types
-        """
-        data = self.to_dict()
-        # Convert fields which need special types not supported by json
-        if data.get('tablespaces') is not None:
-            data['tablespaces'] = [list(item)
-                                   for item in data['tablespaces']]
-        if data.get('begin_time') is not None:
-            data['begin_time'] = data['begin_time'].ctime()
-        if data.get('end_time') is not None:
-            data['end_time'] = data['end_time'].ctime()
-        return data
-
-    @classmethod
-    def from_json(cls, server, json_backup_info):
-        """
-        Factory method that builds a BackupInfo object
-        from a json dictionary
-
-        :param barman.Server server: the server related to the Backup
-        :param dict json_backup_info: the data set containing values from json
-        """
-        data = dict(json_backup_info)
-        # Convert fields which need special types not supported by json
-        if data.get('tablespaces') is not None:
-            data['tablespaces'] = [Tablespace._make(item)
-                                   for item in data['tablespaces']]
-        if data.get('begin_time') is not None:
-            data['begin_time'] = load_datetime_tz(data['begin_time'])
-        if data.get('end_time') is not None:
-            data['end_time'] = load_datetime_tz(data['end_time'])
-        # Instantiate a BackupInfo object using the converted fields
-        return cls(server, **data)
+        super(LocalBackupInfo, self).save(filename=filename, file_object=file_object)
